package jalview.workers;

import java.util.Hashtable;

import jalview.analysis.AAFrequency;
import jalview.analysis.StructureFrequency;
import jalview.api.AlignCalcWorkerI;
import jalview.api.AlignViewportI;
import jalview.api.AlignmentViewPanel;
import jalview.datamodel.AlignmentAnnotation;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.Annotation;

public class StrucConsensusThread extends AlignCalcWorker implements AlignCalcWorkerI
{
  public StrucConsensusThread(AlignViewportI alignViewport,
          AlignmentViewPanel alignPanel)
  {
    super(alignViewport, alignPanel);
  }
  public void run()
  {
    try
    {
      calcMan.notifyStart(this);
      while (!calcMan.notifyWorking(this))
      {
        try
        {
          if (ap != null)
          {
            ap.paintAlignment(false);
          }

          Thread.sleep(200);
        } catch (Exception ex)
        {
          ex.printStackTrace();
        }
      }
      if (alignViewport.isClosed())
      {
        abortAndDestroy();
      }
      AlignmentI alignment = alignViewport.getAlignment();

      int aWidth = -1;

      if (alignment == null || (aWidth = alignment.getWidth()) < 0)
      {
        calcMan.workerComplete(this);
        return;
      }
      AlignmentAnnotation strucConsensus=alignViewport.getAlignmentStrucConsensusAnnotation();
      Hashtable[] hStrucConsensus=alignViewport.getRnaStructureConsensusHash();
      strucConsensus.annotations = null;
      strucConsensus.annotations = new Annotation[aWidth];

      hStrucConsensus = new Hashtable[aWidth];

      AlignmentAnnotation[] aa = alignViewport.getAlignment()
              .getAlignmentAnnotation();
      AlignmentAnnotation rnaStruc = null;
      // select rna struct to use for calculation
      for (int i = 0; i < aa.length; i++)
      {
        if (aa[i].getRNAStruc() != null)
        {
          rnaStruc = aa[i];
          break;
        }
      }
      // check to see if its valid
      
      if (rnaStruc==null || !rnaStruc.isValidStruc())
      {
        calcMan.workerComplete(this);
        return;
      }
      
      jalview.analysis.StructureFrequency.calculate(alignment.getSequencesArray(), 0,
              alignment.getWidth(), hStrucConsensus, true, rnaStruc);
      alignViewport.setRnaStructureConsensusHash(hStrucConsensus);
      // TODO AlignmentAnnotation rnaStruc!!!
      updateResultAnnotation(true);
      if (alignViewport.getGlobalColourScheme()!= null)
      {
        alignViewport.getGlobalColourScheme().setConsensus(hStrucConsensus);
      }

    } catch (OutOfMemoryError error)
    {
      calcMan.workerCannotRun(this);

      // consensus = null;
      // hconsensus = null;
      ap.raiseOOMWarning("calculating RNA structure consensus", error);
    }

    calcMan.workerComplete(this);
    if (ap != null)
    {
      ap.paintAlignment(true);
    }

  }
  /**
   * update the consensus annotation from the sequence profile data using
   * current visualization settings.
   */
  public void updateAnnotation()
  {
    updateResultAnnotation(false);
  }

  public void updateResultAnnotation(boolean immediate)
  {
    AlignmentAnnotation strucConsensus = alignViewport
            .getAlignmentStrucConsensusAnnotation();
    Hashtable[] hStrucConsensus = alignViewport.getRnaStructureConsensusHash();
    if (immediate || !calcMan.isWorking(this) && strucConsensus!=null && hStrucConsensus!=null)
    {
      StructureFrequency.completeConsensus(strucConsensus,
              hStrucConsensus, 0, hStrucConsensus.length,
              alignViewport.getIgnoreGapsConsensus(),
            alignViewport.isShowSequenceLogo());
    }
  }

}